package example;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

/**
 * A simple GlassFish example
 * @author Lyndon Armitage
 */
@WebServlet(name = "UploadFile", urlPatterns = {"/UploadFile"})
public class GlassfishExample extends HttpServlet{
     
    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException,IOException {
	HttpSession session = request.getSession();
	doRequest(session, request, response);
    }
    
    private void doRequest(HttpSession session, HttpServletRequest request, HttpServletResponse response) throws IOException{
	// reset session attributes
	session.setAttribute("href", null);
	session.setAttribute("dir", "");
	
	// make sure we are getting the right kind of content
	boolean isMultipartContent = ServletFileUpload.isMultipartContent(request);
	if(isMultipartContent) {
	    
	    FileItemFactory factory = new DiskFileItemFactory();
	    ServletFileUpload upload = new ServletFileUpload(factory);
	    try {
		
		List<FileItem> fields = upload.parseRequest(request); 
		Iterator<FileItem> it = fields.iterator();
                FileItem fileItem = null; // variable used to hold the file when it is found
		if(!it.hasNext()) {
		    throw new Exception("No fields provided");
		}
		
		while(it.hasNext()) {
		    // loop through each field
		    FileItem field = it.next();
		    if(field.isFormField()) {
			String fieldName = field.getFieldName();
			String fieldValue = field.getString();
			if(fieldName.equals("dir")) {
			    session.setAttribute("dir", fieldValue);
			}
		    }
		    else {
			// this is the file since it is not a form field
			fileItem = field;
		    }
		}
		
		// No file was actually provided
		if(fileItem == null) {
		   throw new Exception("Not a File"); 
		}
		
		// check the file extension is allowed
		if( !(fileItem.getName().endsWith(".pdf")) ){
		    throw new Exception("Not an allowed file type");                                  
		}
		
		// Next we check to see if the path they desired was not empty
		String dir = (String) session.getAttribute("dir");
		if(dir.equals("")) {
		    throw new Exception("No directory provided");
		}
		
		// Now create the directory if it doesn't already exist
		String fileUploadPath = "../fileUploads/" + dir + "/";
		File fileUploadDir = new File(fileUploadPath);
		if(!fileUploadDir.exists()) {
		    fileUploadDir.mkdirs();
		}
		
		// make a valid file name
		String fileName = fileItem.getName().substring(0,fileItem.getName().length() - 4)
				    .replace(" ", "_").replace("%", "_").replace(".","_");
		
		File uploadedFile = new File(fileUploadPath + fileName+".pdf");
		
		fileItem.write(uploadedFile); //save uploaded file
		fileItem.delete(); // remove file item from memory
		
		session.setAttribute("href", "<b>You succesfully uploaded:</b> " + fileName + ".pdf<br /><b>to:</b> " + dir + "/");
	    
	    } catch(Exception e) {
		session.setAttribute("href", e.getMessage());
	    }
	    
	    // let's write our output to another page
	    response.getWriter().println("<!DOCTYPE html>");
	    response.getWriter().println("<html>");
	    response.getWriter().println("<head>");
	    response.getWriter().println("<title>Response</title>");
	    response.getWriter().println("</head>");
	    response.getWriter().println("<body>");
	    response.getWriter().println("<h1>Response</h1>");
	    response.getWriter().println("<p>" + session.getAttribute("href") + "</p>");
	    response.getWriter().println("</body>");
	    response.getWriter().println("</html>");
	}
	
    }
    
     // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
